
require "TestSuiteMethodFactory.rb"

#TestSuite
class TestSuiteFactory

  def initialize(prefix=nil)
    @filePrefix = prefix
    @filePrefix = "Temp" if prefix == nil
    @tests = []
  end

public
  def makeTestSuiteFiles
    getTestList()
    makeHeaderFIle()
    makeCppFIle()
    makeMainFile()
  end

private
  def getTestList
    reg = /(.*Test)\.h$/
    Dir.foreach(Dir.pwd()) { |file|
      if reg =~ file
        @tests.push(TestSuiteMethodFactory.new($1))
      end
    }
  end

  def suiteClassName()
    @filePrefix + "TestSuite"
  end

  def headerFileName()
    suiteClassName() + ".h"
  end

  #make a header file of a TestSuite class
  def makeHeaderFIle()
    return if FileTest.exists?(headerFileName())
    File.open(headerFileName(), "w") {|file|
        writeHeaderFIle(file)
    }
  end
  
  def writeHeaderFIle(file)
    file.puts <<-"end_header"
#ifndef #{suiteClassName().upcase()}_H
#define #{suiteClassName().upcase()}_H

#include <cppunit/TestSuite.h>

class #{suiteClassName()} : public CppUnit::TestSuite
{
public:
    #{suiteClassName()}();
    virtual ~#{suiteClassName()}() {};
};

#endif
    end_header
  end

  #make a cpp file of a TestSuite class
  def makeCppFIle()
    File.open(suiteClassName() + ".cpp", "w") {|file|
        writeCppFile(file)
    }
  end
    
  def writeCppFile(file)
    file.puts <<-"end_includes"
#include <iostream>
#include <cppunit/TestCaller.h>
#include \"#{headerFileName()}\"
    end_includes
    
    #macro
    TestSuiteMethodFactory::writeSuiteMethodMacro(file)
    
    #write suite methods
    @tests.each { |aClass|
      aClass.writeSuiteMethod(file)
    }
    
    file.puts ''
    file.puts "#{suiteClassName()}::#{suiteClassName()}()"
    file.puts '{'
    
    @tests.each { |aClass|
       aClass.writeAddSuite(file)
    }
    file.puts "}"
  end

  #write a main function
  def makeMainFile()
    File.open("Tester.cpp", "w") {|file|
        writeMainFile(file)
    }
  end
  
  def writeMainFile(file)
    file.puts <<-"end_main"
#include <cppunit/TextTestRunner.h>
#include \"#{headerFileName()}\"

int main() {
    #{suiteClassName()} suite;
    CppUnit::TextTestRunner runner;
    runner.addTest(&suite);
    runner.run();
    return 0;
}
    end_main
  end
end



