
class TestSuiteMethodFactory

    def initialize(classname)
        @classname = classname
        @testMethods = nil
      end

    def writeAddSuite(file)
        if (hasSuiteMethod?)
             file.print("    addTest(" + @classname + "::suite());\n")
            
        else
             file.print("    addTest(" + @classname + "_suite());\n")
        end
    end

    def TestSuiteMethodFactory::writeSuiteMethodMacro(file)
        file.puts <<-"end_macro"

#define TEST_SUITE(ATestCaseType)                               \\
CppUnit::Test* ATestCaseType##_suite() {                        \\
    typedef ATestCaseType __ThisTestCaseType;                   \\
    typedef CppUnit::TestCaller<ATestCaseType> caller_type;     \\
    
    CppUnit::TestSuite* suite = new CppUnit::TestSuite("ATestCaseType");
#define TEST(testMethod)                                        \\
    suite->addTest(new caller_type("testMethod", &__ThisTestCaseType::##testMethod));
#define TEST_SUITE_END()                                        \\
    return suite;                                               \\
}
        end_macro
    end

    def writeSuiteMethod(file)
        file.puts "\n"
        file.puts "#include \"" + @classname + ".h\""
        return if hasSuiteMethod?
        file.puts "TEST_SUITE(" + @classname + ");"
        testMethods.each{ |method|
            file.puts "TEST(" + method + ");";
        }
        file.puts "TEST_SUITE_END();"
    end

    #If this class has a suite() method, return true.
    def hasSuiteMethod?
        @inComments = false
        result = File.new(@classname + ".h").detect { |line|
            suiteMethod?(line)
        }
        result != nil
    end

    #return an array of test methods.
    def testMethods
        parse if @testMethods == nil
        @testMethods
    end

private
    def parse
        @inComments = false
        @testMethods = []
        lex = /\s*void\s+(test.*)\(\)/
        File.foreach(@classname + ".h") { |line|
            unless commentLine?(line)
                @testMethods.push($1) if lex =~ line
            end
        }
    end

    #Return true if the line is in a comments.
    def commentLine?(line)
        #skip // comment
        return true if (/^\s*\/\// =~ line)

        #skip /* */ comment
        return true if (/^\s*\/\*.*\*\// =~ line)

        #/* is the start of a comment
        if (/^\s*\/\*/ =~ line)
            @inComments = true
            return true
        end
        #*/ is an end of a comment
        if (@inComments && /\*\// =~ line)
            @inComments = false
            return true
        end
        return true if @inComments

        false
    end
    
    #Return true if the line is a definition of a suite method.
    def suiteMethod?(line)
        return false if commentLine?(line)
        return true if (/\bstatic\s+Test\s*\*\s*suite\s*\(\s*\)/ =~ line)
        return true if (/CPPUNIT_TEST_SUITE/ =~ line)
    end


end

